
# Installation d'une carte satellite

Voici deux méthodes décrivant comment générer et rendre accessible une carte satellite de Minetest NALC.  
La première méthode avec **MTSatellite** est la plus coûteuse en ressource mais permet d'obtenir une carte qui se renouvelle dynamiquement. L'inconvénient est qu'il faut convertir le backend de la map vers un format spécifique.  
La seconde méthode avec **minetestmapper** permet de généner un instantané de la carte mais n'est pas dymique. L'aventage est que la solution est peu couteuse en ressources et compatible avec le backend en PostgreSQL de la map de NALC.


## Installation avec MTSatellite


### Précautions

Les instructions suivantes peuvent mener à la destruction du serveur Minetest. Soyez sûr d'avoir effectué des sauvegardes au préalable.  
Ces instructions sont destinées au serveur Minetest NALC.


### Installation de MTSatellite (Gentoo)

Se connecter en utilisateur root sur le serveur :

    -- Installation des packages nécessaires
    # emerge -av go leveldb mercurial

Ensuite suivre les [instructions de compilations](https://sys4.fr/gitea/mtcontrib/mtsatellite/src/branch/master/COMPILE.md) de mtsatellite.


<a id="orgf9c3427"></a>

### Création du fichier colors.txt

Je recommande si possible de faire cette opération sur un serveur de test avec la même liste de mods que le serveur de production.

Se placer à la racine du serveur minetest et télécharger le dépôt MTAutocolors :   

    $ cd ~/nalc-test
    $ git clone --depth 1 https://sys4.fr/gitea/nalc/MTAutocolors.git

Les instructions d'installation du dépôt [MTAutocolors](https://sys4.fr/gitea/nalc/MTAutocolors) sont obsolètes car le dépôt officiel n'existe plus. Voici donc comment installer MTAutocolors :

    $ cd ~/nalc-test/MTAutocolors/cmd/mtautocolors
    $ go mod init minetest/mtautocolors
    $ go mod tidy
    $ go get -u minetest/mtautocolors

L'exécutable mtautocolors est maintenant installé dans `~/go/bin`.

Créer un lien symbolique du mod automappercolors dans le dossier `custom/mods` :

    $ cd ~/nalc-test
    $ mkdir -p custom/mods
    $ ln -s ~/nalc-test/MTAutocolors/mods/automappercolors custom/mods/

Mise à jour de la liste des mods que le serveur doit charger au démarrage :

    $ ./shutdown.sh # Éteindre le serveur
    $ ./upgrade.sh -b stable -w nalc-stable-test -f worldmt # Changer la branche et le nom du world si nécessaire
    $ ./shutdown.sh -r # Relancer le serveur

Maintenant suivre les instructions du paragraphe **usage** du fichier `~/nalc-test/MTAutocolors/README.md` pour pouvoir générer le fichier `colors.txt`.


### Migrer le backend de la map vers levelDB ou SQLite3

À faire uniquement si le backend actuel est différent.

    $ cd ~/nalc-stable/minetest/bin
    $ ./minetestserver --migrate leveldb --world /home/minetest/nalc-stable/minetest/worlds/nalc-stable

Remplacer `leveldb` par `sqlite` si besoin.


### Configurer MTSatellite

Suivre [les instructions de configuration](https://sys4.fr/gitea/mtcontrib/mtsatellite/src/branch/master/SETUP.md) du dépôt.


## Installation avec minetestmapper

Tout d'abord il faut installer le langage go si ce n'est pas déjà fait :

    -- Installation des packages nécessaires
    # emerge -av go leveldb mercurial

Puis procéder à la [Création du fichier colors.txt](#orgf9c3427) de la même façon que pour l'installation avec MTSatellite.


### Téléchargement et compilation

Se placer à la racine du serveur NALC et cloner le dépôt mirroir :

    $ cd ~/nalc-stable
    $ git clone --depth 1 https://sys4.fr/gitea/minetest/minetestmapper.git

Suivre les instructions du fichier `~/nalc-stable/minetestmapper/README.rst` et installer les dépendances manquantes.

Lancer la compilation :

    $ cmake .
    $ make -j$(nproc)


### Générer la carte

    $ cp ~/nalc-stable/minetestmapper
    $ ./minetestmapper -i ~/nalc-stable/minetest/worlds/nalc-stable/ -o nalc_map.png --colors ~/nalc-test/colors.txt --min-y -25 --max-y 300 --backend postgresql --geometry -5000:-5000+10000+10000

